---------------------------------------------------------------
--                                                           
--  PENCIL_AND_PAPER_COMPUTER INTERPRETER
--  By: Martin Carlisle and Al White
--                                                           
--  PENCIL AND PAPER COMPUTER 
--  Description : non-GUI portion of Pencil and Paper Computer
--                interpreter 
--                       
--  By: Al White
--                                         
-- PENCIL AND PAPER COMPUTER INTERPRETER is free software; 
-- you can redistribute it and/or modify it without restriction.  
-- However, we ask that you please retain the original author 
-- information, and clearly indicate if it has been modified.
--  
-- PENCIL AND PAPER COMPUTER INTERPRETER is distributed in the 
-- hope that it will be useful, but WITHOUT ANY WARRANTY; without 
-- even the implied warranty of MERCHANTABILITY or FITNESS FOR 
-- A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
package Pencil_And_Paper_Computer is
   subtype Accumulator_Value is Integer range 0..2**14-1;
   subtype Memory_Value is Integer range 0..255;
   subtype Memory_Location is Integer range 0..31;

   --------------------------------------------------
   -- sets a particular memory location to a value
   --------------------------------------------------
   procedure Set_Location (
         Address : in     Memory_Location; 
         Value   : in     Memory_Value     ); 

   --------------------------------------------------
   -- gets a particular memory location, either as an
   -- integer, or as a string (e.g. "ADD 13")
   --------------------------------------------------
   function Get_Location (
         Address : in     Memory_Location ) 
     return Memory_Value; 
   function Get_Location (
         Address : in     Memory_Location ) 
     return String; 

   --------------------------------------------------
   -- gets current value of the accumulator
   --------------------------------------------------
   function Get_Accumulator return Accumulator_Value; 
   procedure Set_Accumulator(Value : in Accumulator_Value);

   --------------------------------------------------
   -- gets current value of the PC
   --------------------------------------------------
   function Get_Program_Counter return Memory_Location; 
   procedure Set_Program_Counter(Address : in Memory_Location);

   --------------------------------------------------
   -- sets PC to 0, and "starts" the program
   -- accumulator is reset to 0.
   --------------------------------------------------
   procedure Reset_Program_Counter; 

   --------------------------------------------------
   -- Perform the next instruction.  
   --
   -- Each time memory
   -- is changed, Update_Callback will be called with
   -- the address of the changed memory location.
   --
   -- If a read instruction is done, then
   -- the Read_Callback is used to get the value
   -- from the user.
   --
   -- If a print instruction is done, then
   -- the Print_Callback is used to print the value to
   -- the computer screen
   --
   -- raises Program_Halted if the program hasn't
   -- been started or has halted.
   --------------------------------------------------

   type post_read_callback_type is access procedure(
         Readvalue : in     Memory_Value); 

   type Update_Callback_Type is access procedure 
   (
      Location : in Memory_Location);
   type Read_Callback_Type is access procedure 
   (X: post_read_callback_type);
   type Print_Callback_Type is access procedure 
   (X : in Memory_Value);
   Program_Halted   : exception;  
   Memory_Overflow  : exception;  
   Memory_Underflow : exception;
   procedure Do_One_Instruction (
         Update_Callback : in     Update_Callback_Type; 
         Read_Callback   : in     Read_Callback_Type;   
         Print_Callback  : in     Print_Callback_Type   ); 

   --------------------------------------------------
   -- read a program (contents of memory) from a file
   -- may raise an exception if filename doesn't exist
   -- or is improperly formatted.
   --------------------------------------------------
   procedure Read_Program (
         Filename : in     String ); 

   --------------------------------------------------
   -- write a program (contents of memory) to a file
   -- may raise an exception only if the filename
   -- specified cannot be created.
   --------------------------------------------------
   procedure Write_Program (
         Filename : in     String ); 
         
   Stop_Instruction : constant Memory_Value := 0;
   
   -- shift instructions
   procedure Insert_Instruction;
   procedure Delete_Instruction;
private
   type Instruction is 
         (Stop,  
          Add,   
          Sub,   
          Load,  
          Store, 
          Read,  
          Print, 
          Pjump); 
end Pencil_And_Paper_Computer;